#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun  7 16:39:28 2023

@author: schoelleh96
"""

# %% library imports

# This will move the console to the right working directory.
from os.path import  dirname, abspath, exists
from os import chdir, makedirs
chdir(dirname(abspath(__file__)))

from sys import argv
import numpy as np
import plot as pp
import calc as cc
import data as dd
import matplotlib.pyplot as plt
from datetime import datetime
from matplotlib.widgets import (Slider, CheckButtons, RadioButtons,
                                RangeSlider, Button)

# %% Get the data

startkp = True

date_0 = datetime(2016,5,1,6)

# only every nth trajectory
for n in [1, 2, 5, 10]:

    if startkp:
        fname = date_0.strftime('trajkp_%Y%m%d_%H') + ".npy"
        path = "./" + date_0.strftime("datkp_%Y%m%d_%H") + "/" + str(n) + "/"
    else:
        fname = date_0.strftime('traj_%Y%m%d_%H') + ".npy"
        path = "./" + date_0.strftime("dat_%Y%m%d_%H") + "/" + str(n) + "/"

    trajs = np.load(fname)

    lon = trajs['lon'][::n]
    lat = trajs['lat'][::n]
    p = trajs['p'][::n]
    u = trajs['U'][::n]
    v = trajs['V'][::n]
    omg = trajs['OMEGA'][::n]
    T = trajs['T'][::n]

    # %% Initials

    alpha = 0.1

    H = cc.ptoh(p, 1013.25, 8.435) # standard atmosphere

    if not exists(path):
        makedirs(path)

    # %% loop over coordinates

    for coord in ["Lon, Lat, p", "Lon, Lat, h", "x, y, p (stereo)",
                  "x, y, h (stereo)","x, y, z (3d)"]:
        if coord == "Lon, Lat, p":
            x, y, z = lon, lat, p
            d_path = path + "lonlatp"
        elif coord == "Lon, Lat, h":
            x, y = lon, lat
            z = cc.ptoh(p, 1013.25, 8.435)
            d_path = path + "lonlath"
        elif coord == "x, y, p (stereo)":
            x, y, z = cc.coord_trans(lon, lat, p, "None", 1, proj="stereo")
            d_path = path + "stereop"
        elif coord == "x, y, h (stereo)":
            x, y, z = cc.coord_trans(lon, lat, p, "std_atm", 1, proj="stereo")
            d_path = path + "stereoh"
        elif coord == "x, y, z (3d)":
            x, y, z = cc.coord_trans(lon, lat, p, "std_atm", -1)
            d_path = path + "xyz"

        for norm in [True, False]:
            # Normalization
            if norm:
                x, y, z = cc.norm(x), cc.norm(y), cc.norm(z)
                d_path = d_path + "norm"

            for meth in ["Convex", "$\\alpha$", "opt. $\\alpha$"]:
                if meth == "$\\alpha$":
                    for alpha in [0.01, 0.05, 0.1, 0.5, 1, 5, 10]:
                        # Boundary calculation
                        bounds, hulls = dd.io_bounds(d_path, meth, x, y,
                                             z, alpha)
                else:
                    bounds, hulls = dd.io_bounds(d_path, meth, x, y,
                                                 z, alpha)

