#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 17 15:48:17 2023

@author: schoelleh96
"""

# %% library imports

# This will move the console to the right working directory.
from os.path import  dirname, abspath, exists
from os import chdir, makedirs
chdir(dirname(abspath(__file__)))

from sys import argv
import numpy as np
import plot as pp
import calc as cc
import data as dd
import matplotlib.pyplot as plt
from datetime import datetime
from matplotlib.widgets import (Slider, CheckButtons, RadioButtons,
                                Button)
from warnings import catch_warnings, simplefilter

# %% Get the data

startkp = False

date_0 = datetime(2016,5,1,6)

# only every nth trajectory
n = 5

if startkp:
    fname = date_0.strftime('trajkp_%Y%m%d_%H') + ".npy"
    path = "./" + date_0.strftime("datkp_%Y%m%d_%H") + "/" + str(n) + "/"
else:
    fname = date_0.strftime('traj_%Y%m%d_%H') + ".npy"
    path = "./" + date_0.strftime("dat_%Y%m%d_%H") + "/" + str(n) + "/"

trajs = np.load(fname)

if not exists(path):
    makedirs(path)

lon = trajs['lon'][::n]
lat = trajs['lat'][::n]
p = trajs['p'][::n]
u = trajs['U'][::n]
v = trajs['V'][::n]
omg = trajs['OMEGA'][::n]
T = trajs['T'][::n]

# %% Initials

azim = 150
elev = 45
dist = 7
t_i = 0
alpha = 0.1
e = 50000
epsilon_opt = np.array([25000, 50000, 100000, 250000, 500000])
N_k = 3
# u and v is in m/s, omg in P/s
k_p = cc.calc_k(u/1000, v/1000, omg/100)

# Coordinates for visualization
X = lon
Y = lat
Z = p
H = cc.ptoh(p, 1013.25, 8.435) # standard atmosphere
k_h = cc.calc_k(u, v, cc.omg2w(omg, T, p))

# %% Boundary

if argv[0] == '' or argv[1] == "Boundary":

    # Initialize

    fig_bound = plt.figure(figsize=(14,8))#, constrained_layout=True)
    ax3d = fig_bound.add_axes([0.2, 0.025, 0.8, 0.9], projection="3d")

    bounds, hulls = dd.io_bounds(path + "lonlatp", "Convex", lon, lat, p, None)

    ax3d.view_init(azim=azim, elev=elev)
    with catch_warnings():
        simplefilter("ignore")
        ax3d.dist=dist
    ax3d.scatter(X[:,t_i], Y[:,t_i], Z[:,t_i], c=bounds[:, t_i])

    ax3d.set_xlabel("Lon")
    ax3d.set_ylabel("Lat")
    ax3d.set_zlabel("p")

    # Create Interactives
    t_slider_ax  = fig_bound.add_axes([0.05, 0.95, 0.15, 0.025])
    t_slider = Slider(t_slider_ax, 'T', 0, lon.shape[1]-1, valinit=t_i,
                      valstep=1)
    dist_slider_ax  = fig_bound.add_axes([0.05, 0.92, 0.15, 0.025])
    dist_slider = Slider(dist_slider_ax, 'Zoom', 0, 10, valinit=dist)
    alph_slider_ax  = fig_bound.add_axes([0.05, 0.89, 0.15, 0.025])
    alph_slider = Slider(alph_slider_ax, "$\log(\\alpha)$", -4, 2,
                         valinit=np.log10(alpha))
    delt_slider_ax  = fig_bound.add_axes([0.05, 0.86, 0.15, 0.025])
    delt_slider = Slider(delt_slider_ax, "$\Delta$", 0, 1, valinit=0)

    meth_ax = fig_bound.add_axes([0.02, 0.76, 0.15, 0.07])
    meth_check = RadioButtons(meth_ax, ["Convex", "$\\alpha$",
                                        "opt. $\\alpha$",  "$\Delta$"], 0)
    meth_ax.set_title("Hull Method")

    coord_ax = fig_bound.add_axes([0.02, 0.63, 0.15, 0.1])
    coord_check = RadioButtons(coord_ax, ["Lon, Lat, p", "Lon, Lat, h",
                                          "x, y, p (stereo)",
                                          "x, y, h (stereo)",
                                          "x, y, z (3d)"], 0)
    coord_ax.set_title("Coordinates")

    norm_ax = fig_bound.add_axes([0.02, 0.56, 0.15, 0.05])
    norm_check = CheckButtons(norm_ax, ["normalize?"], [False])

    run_ax = fig_bound.add_axes([0.02, 0.5, 0.15, 0.05])
    run_check = Button(run_ax, "Calculate")

    hull_ax = fig_bound.add_axes([0.02, 0.25, 0.15, 0.05])
    hull_check = CheckButtons(hull_ax, ["Plot hull?"], [False])

    vis_ax = fig_bound.add_axes([0.02, 0.1, 0.15, 0.1])
    vis_check = RadioButtons(vis_ax, ["Lon, Lat, p", "Lon, Lat, h",
                                      "x, y, p (stereo)",
                                      "x, y, h (stereo)",
                                      "x, y, z (3d)"], 0)
    vis_ax.set_title("Visualization")

    plot_ax = fig_bound.add_axes([0.02, 0.02, 0.15, 0.05])
    plot_check = Button(plot_ax, "Plot")

    # Make sliders interactive (no need to recalculate)
    def slider_changed(val):
        global X, Y, Z, bounds, hulls

        azim_i = ax3d.azim
        elev_i = ax3d.elev
        ax3d.clear()

        ax3d.view_init(azim=azim_i, elev=elev_i)
        with catch_warnings():
            simplefilter("ignore")
            ax3d.dist=dist_slider.val

        ax3d.scatter(X[:,t_slider.val], Y[:,t_slider.val], Z[:,t_slider.val],
                     c=bounds[:, t_slider.val])

        ax3d.set_xlabel(vis_check.value_selected.split(", ")[0])
        ax3d.set_ylabel(vis_check.value_selected.split(", ")[1])
        ax3d.set_zlabel(vis_check.value_selected.split(", ")[2].split(" ")[0])

        if hull_check.get_status()[0]:
            ax3d.plot_trisurf(hulls[t_slider.val].vertices[:, 0],
                              hulls[t_slider.val].vertices[:,1],
                              triangles=hulls[t_slider.val].faces,
                              Z=hulls[t_slider.val].vertices[:,2],
                              alpha=0.5)

    dist_slider.on_changed(slider_changed)
    t_slider.on_changed(slider_changed)

    # Make calculate button interactive
    def calc(val):
        global bounds, lon, lat, p, hulls

        # Coordinates for boundary calculation
        if coord_check.value_selected == "Lon, Lat, p":
            x, y, z = lon, lat, p
            d_path = path + "lonlatp"
        elif coord_check.value_selected == "Lon, Lat, h":
            x, y = lon, lat
            z = cc.ptoh(p, 1013.25, 8.435)
            d_path = path + "lonlath"
        elif coord_check.value_selected == "x, y, p (stereo)":
            x, y, z = cc.coord_trans(lon, lat, p, "None", 1, proj="stereo")
            d_path = path + "stereop"
        elif coord_check.value_selected == "x, y, h (stereo)":
            x, y, z = cc.coord_trans(lon, lat, p, "std_atm", 1, proj="stereo")
            d_path = path + "stereoh"
        elif coord_check.value_selected == "x, y, z (3d)":
            x, y, z = cc.coord_trans(lon, lat, p, "std_atm", -1)
            d_path = path + "xyz"

        # Normalization
        if norm_check.get_status()[0]:
            x, y, z = cc.norm(x), cc.norm(y), cc.norm(z)
            d_path = d_path + "norm"

        # Boundary calculation
        bounds, hulls = dd.io_bounds(d_path, meth_check.value_selected, x, y,
                                     z, 10**(alph_slider.val))

        print("Calculation Done")

        # If there is a time step, where all points are boundary
        if (bounds == 1).all(axis=0).any():
            allb = np.where((bounds == 1).all(axis=0))
            print("All points boundary at time {}; ".format(allb) +
                   "please normalize or adjust $\\alpha$")

    run_check.on_clicked(calc)

    # Make visualization button interactive
    def plot(val):
        global bounds, X, Y, Z, lon, lat, p, hulls

        if vis_check.value_selected == "Lon, Lat, p":
            X, Y, Z = lon, lat, p
        elif vis_check.value_selected == "Lon, Lat, h":
            X, Y = lon, lat
            Z = cc.ptoh(p, 1013.25, 8.435)
        elif vis_check.value_selected == "x, y, p (stereo)":
            X, Y, Z = cc.coord_trans(lon, lat, p, "None", 1, proj="stereo")
        elif vis_check.value_selected == "x, y, h (stereo)":
            X, Y, Z = cc.coord_trans(lon, lat, p, "std_atm", 1, proj="stereo")
        elif vis_check.value_selected == "x, y, z (3d)":
            X, Y, Z = cc.coord_trans(lon, lat, p, "std_atm", -1)

        # Normalization
        if norm_check.get_status()[0]:
            X, Y, Z = cc.norm(X), cc.norm(Y), cc.norm(Z)

        slider_changed(None)
        fig_bound.canvas.draw_idle()
        print("Plotting Done")

    plot_check.on_clicked(plot)

# %% Coherent Sets

# elif argv[0] == '' or argv[1] == "Sets":
if True:
    # Initialize
    fig_cs = plt.figure(figsize=(14,8))#, constrained_layout=True)
    ax3dcs = fig_cs.add_axes([0.2, 0.025, 0.8, 0.9], projection="3d")
    ax_spect = fig_cs.add_axes([0.03, 0.025, 0.3, 0.3])

    x, y, z = cc.coord_trans(lon, lat, p, "None", proj="stereo",
                             scaling = 1)
    x, y, z = cc.norm(x), cc.norm(y), cc.norm(z)

    bounds, hulls = dd.io_bounds(path + "stereop" + "norm" ,
                                 "opt. $\\alpha$", x, y, z, None)

    bound_meth = "Concave"

    pp.plot_spectra(3*np.sqrt(epsilon_opt), epsilon_opt, lon, lat, p,
                    k_p, ax_spect, bounds, True, path, "p")

    D = dd.io_dist(path, e, lon, lat, p, True, k_p, "p")

    E = cc.eigen(lon.shape[0], 0, lon.shape[1],
                          D[0], D[1], D[2], e, bounds=bounds)
    kclust = cc.kcluster_idx(E, N_k)

    Xcs, Ycs, Zcs = cc.coord_trans(lon, lat, p, "None", 1, proj="stereo")

    points = pp.plot_clustering(ax3dcs, X, Y, Z, azim, elev, dist, t_i,
                                c=kclust.labels_, plot_0=False, coord="stereo")
    ax3dcs.invert_zaxis()

    # Add widget items
    t_slider_ax  = fig_cs.add_axes([0.06, 0.95, 0.2, 0.025])
    t_slider = Slider(t_slider_ax, 'T', 0, lon.shape[1]-1, valinit=t_i,
                      valstep=1)
    dist_slider_ax  = fig_cs.add_axes([0.06, 0.92, 0.2, 0.025])
    dist_slider = Slider(dist_slider_ax, 'Zoom', 0, 10, valinit=dist)
    eps_slider_ax  = fig_cs.add_axes([0.06, 0.89, 0.2, 0.025])
    eps_slider = Slider(eps_slider_ax, '$\log(\epsilon)$ in $km^2$',
                        4, 6, valinit=np.log10(e))
    N_k_ax = fig_cs.add_axes([0.06, 0.86, 0.2, 0.025])
    N_k_slider = Slider(N_k_ax, "N_k", 2,20, valinit=N_k, valstep=1)
    sc_ax = fig_cs.add_axes([0.02, 0.7, 0.1, 0.1])
    sc_check = RadioButtons(sc_ax, ["$k_p$", "$k_h$", "sd units"], 0)
    sc_ax.set_title("Vertical Scaling")
    bound_ax = fig_cs.add_axes([0.02, 0.55, 0.1, 0.1])
    bound_check = RadioButtons(bound_ax, ["Concave", "Convex"], 0)
    bound_ax.set_title("Hull Method")
    run_ax = fig_cs.add_axes([0.02, 0.45, 0.1, 0.05])
    run_check2 = Button(run_ax, "Calculate")

    # Make sliders functional
    def slider_changed_cs(val):
        global dat, v, points, colors, vals, inv
        azim_i = ax3dcs.azim
        elev_i = ax3dcs.elev
        ax3dcs.clear()

        points = pp.plot_clustering(ax3dcs, Xcs, Ycs, Zcs, azim_i, elev_i,
                                    dist_slider.val, t_slider.val,
                                    c=kclust.labels_, plot_0=True,
                                    coord="stereo")
        ax3dcs.invert_zaxis()

        for c in np.unique(kclust.labels_):
            colors[c] = points.to_rgba(c)

        keys, inv = np.unique(kclust.labels_, return_inverse=True)
        vals = np.array([colors[key] for key in keys])

        var_changed(None)

        fig_cs.canvas.draw_idle()

    dist_slider.on_changed(slider_changed_cs)
    t_slider.on_changed(slider_changed_cs)

    # Make N_k slider functional
    def N_k_slider_changed(val):
        global E, kclust
        kclust = cc.kcluster_idx(E, N_k_slider.val)
        slider_changed_cs(None)

    N_k_slider.on_changed(N_k_slider_changed)

    # Make Calculation Button functional
    def calc_cs(val):
        global bounds, hulls, D, E, kclust, Xcs, Ycs, Zcs, colors, vals, bound_meth

        if ((bound_check.value_selected == "Concave") &
            (bound_meth != "Concave")):
            bounds, hulls = dd.io_bounds(path + "stereop" + "norm" ,
                                         "opt. $\\alpha$", x, y, z, None)
            bound_meth = "Concave"

        elif ((bound_check.value_selected == "Convex") &
              (bound_meth != "Convex")):
            bounds, hulls = dd.io_bounds(path + "stereop" + "norm" ,
                                         "Convex", x, y, z, None)

            bound_meth = "Convex"

        if sc_check.value_selected == "$k_p$":
            pp.plot_spectra(3*np.sqrt(epsilon_opt), epsilon_opt, lon, lat, p,
                            k_p, ax_spect, bounds, True, path,
                            "p")
            D = dd.io_dist(path, 10**eps_slider.val, lon, lat, p,
                           True, k_p, "p")

        elif sc_check.value_selected() == "$k_h$":
            pp.plot_spectra(3*np.sqrt(epsilon_opt), epsilon_opt, lon, lat, H,
                            k_h, ax_spect, bounds, True, path, "H")
            D = dd.io_dist(path, 10**eps_slider.val, lon, lat, H,
                           True, k_h, "H")

        E = cc.eigen(lon.shape[0], 0, lon.shape[1],
                     D[0], D[1], D[2], 10**eps_slider.val,
                     bounds=bounds)
        kclust = cc.kcluster_idx(E, N_k_slider.val)

        slider_changed_cs(None)

    run_check2.on_clicked(calc_cs)

     # %% spaghetti

    alpha_spag = 0.1 + (n/100)

    fig_spag = plt.figure()
    ax_spag = fig_spag.add_subplot(111)
    fig_spag.subplots_adjust(left=0.2)
    var_ax = fig_spag.add_axes([0.025, 0.1, 0.1, 0.8])
    var_check = RadioButtons(var_ax, trajs.dtype.names, 3)
    var_ax.set_title("Variable")
    dat = trajs[var_check.value_selected][::n]

    colors = {}
    for c in np.unique(kclust.labels_):
        colors[c] = points.to_rgba(c)

    keys, inv = np.unique(kclust.labels_, return_inverse=True)
    vals = np.array([colors[key] for key in keys])
    pp.plot_spaghetti(ax_spag, dat, alpha_spag, vals[inv])

    v = pp.draw_vert_t(ax_spag, 0, lon.shape[1], t_slider.val)
    plt.show()

    def var_changed(val):
        dat = trajs[var_check.value_selected][::n]
        ax_spag.clear()
        pp.plot_spaghetti(ax_spag, dat, alpha_spag, vals[inv])
        pp.draw_vert_t(ax_spag, 0, lon.shape[1], t_slider.val)
        fig_spag.canvas.draw_idle()

    var_check.on_clicked(var_changed)
